@@ -1,4 +1,7 @@
 use itertools::Itertools;
+
+use rinex::Type;
+use rinex::epoch;
 use rinex::constellation::Constellation;
 
 fn main() {
@@ -8,15 +11,15 @@ fn main() {
 
     // example file
     let navigation_file = std::path::PathBuf::from(
-        env!("CARGO_MANIFEST_DIR").to_owned() + "/data/CBW100NLD_R_20210010000_01D_MN.rnx");
+        env!("CARGO_MANIFEST_DIR").to_owned() + "/data/NAV/V3/CBW100NLD_R_20210010000_01D_MN.rnx");
     // parse example file
     let rinex = rinex::Rinex::from_file(&navigation_file).unwrap();
 
     // header information
     assert_eq!(rinex.header.is_crinex(), false);
-    assert_eq!(rinex.header.rinex_type, rinex::Type::NavigationMessage);
+    assert_eq!(rinex.header.rinex_type, Type::NavigationMessage);
     assert_eq!(rinex.header.version.major, 3);
-    assert_eq!(rinex.header.constellation, Constellation::Mixed); 
+    assert_eq!(rinex.header.constellation, Some(Constellation::Mixed)); 
     // leap second field for instance
     // is a major > 3 optionnal field
     assert_eq!(rinex.header.leap.is_some(), true);
@@ -50,18 +53,19 @@ fn main() {
     // match a specific `epoch`
     //  * `epoch` is a chrono::NaiveDateTime alias
     //     therefore one can use any chrono::NaiveDateTime method
-    let to_match = rinex::epoch::from_string("21 01 01 08 45 00")
-        .unwrap();
+    let to_match = epoch::Epoch::new(
+        epoch::str2date("21 01 01 00 00 00").unwrap(),
+        epoch::EpochFlag::default());
     //    ---> retrieve all data for desired `epoch`
     //         using direct hashmap[indexing]
     let matched = &record[&to_match];
     println!("\n------------- Matching epoch \"{:?}\" ----------\n{:#?}", to_match, matched); 
     
-    // ----> zoom in on `R24` vehicule for that particular `epoch` 
-    let to_match = rinex::record::Sv::new(Constellation::Glonass, 24);
+    // ----> zoom in on `E01` vehicule for that particular `epoch` 
+    let to_match = rinex::record::Sv::new(Constellation::Galileo, 1);
     let matched = &matched[&to_match];
     println!("\n------------- Adding Sv filter \"{:?}\" to previous epoch filter ----------\n{:#?}", to_match, matched); 
-    // ----> zoom in on `B07` clock drift for that `epoch`
+    // ----> zoom in on `E01` clock drift for that `epoch`
     let matched = &matched["ClockDrift"];
     println!("\n------------- \"clockDrift\" data from previous set ----------\n{:#?}", matched); 
     
@@ -78,6 +82,7 @@ fn main() {
     // list all epochs
     let epochs: Vec<_> = record
         .keys()
+        .map(|k| k.date)
         .sorted()
         .collect();
     println!("\n------------- Epochs ----------\n{:#?}", epochs); 
