@@ -1,16 +1,15 @@
 //! Describes a `RINEX` header, includes
 //! rinex header parser and associated methods
-//use regex::Regex;
 use thiserror::Error;
+use std::collections::HashMap;
 
 use crate::version;
 use crate::gnss_time;
 use crate::{is_comment, Type, TypeError};
+use crate::constellation;
 use crate::constellation::{Constellation, ConstellationError};
-use crate::meteo::{MeteoDataCode, MeteoDataCodeError};
-use crate::observation::{ObservationCode, ObservationCodeError};
 
-/// Describes a `CRINEX` (compact rinex) 
+/// Describes a `CRINEX` (compressed rinex) 
 pub const CRINEX_MARKER_COMMENT : &str = "COMPACT RINEX FORMAT";
 /// End of Header section reached
 pub const HEADER_END_MARKER : &str = "END OF HEADER";
@@ -48,6 +47,32 @@ impl std::str::FromStr for Rcvr {
     }
 }
 
+/// Meteo Observation Sensor
+#[derive(Debug)]
+pub struct Sensor {
+	/// Model of this sensor
+	model: String,
+	/// Type of sensor
+	sens_type: String,
+	/// Sensor accuracy [°C,..]
+	accuracy: f32,
+	/// Physics measured by this sensor
+	physics: String,
+}
+
+impl Sensor {
+	/// Builds a new Meteo Obs sensor,
+	/// with given `model`, `sensor type` `accuracy` and `physics` fields
+	pub fn new (model: &str, sens_type: &str, accuracy: f32, physics: &str) -> Sensor {
+		Sensor {
+			model: String::from(model),
+			sens_type: String::from(sens_type),
+			accuracy,
+			physics: String::from(physics),
+		}
+	}
+}
+
 /// Antenna description 
 #[derive(Debug)]
 pub struct Antenna {
@@ -205,13 +230,14 @@ enum MarkerType {
 pub struct RinexHeader {
     /// revision for this `RINEX`
     pub version: version::Version, 
-    /// optionnal `CRINEX` (compact `RINEX` infos), 
+    /// optionnal `CRINEX` (compressed `RINEX` infos), 
     /// if this is a CRINEX
     pub crinex: Option<CrinexInfo>, 
     /// type of `RINEX` file
     pub rinex_type: Type, 
-    /// `GNSS` constellation being used
-    pub constellation: Constellation, 
+    /// specific `GNSS` constellation system,
+	/// may not exist for RINEX files 
+    pub constellation: Option<Constellation>, 
     /// program name
     pub program: String, 
     /// program `run by`
@@ -230,6 +256,8 @@ pub struct RinexHeader {
     pub rcvr: Option<Rcvr>, 
     /// optionnal antenna infos
     pub ant: Option<Antenna>, 
+	/// optionnal meteo sensors infos
+	pub sensors: Option<Vec<Sensor>>,
     /// optionnal leap seconds infos
     pub leap: Option<LeapSecond>, 
     /// station approxiamte coordinates
@@ -255,16 +283,19 @@ pub struct RinexHeader {
     //gnsstime_corr: Option<Vec<gnss_time::GnssTimeCorr>>,
     /// Observations:   
     /// true if epochs & data compensate for local clock drift 
-    pub rcvr_clock_offset_applied: Option<bool>, 
+    pub rcvr_clock_offset_applied: bool, 
     // observation (specific)
     /// lists all types of observations 
     /// contained in this `Rinex` OBS file
-    pub obs_codes: Option<Vec<String>>, 
+    pub obs_codes: Option<HashMap<Constellation, Vec<String>>>, 
+	/// lists all types of observations
+	/// contains in this `RINEX` Meteo file
+    pub met_codes: Option<Vec<String>>, 
 }
 
 #[derive(Error, Debug)]
 pub enum Error {
-    #[error("Compact RINEX related content mismatch")]
+    #[error("CRINEX related content mismatch")]
     CrinexFormatError,
     #[error("RINEX version is not supported '{0}'")]
     VersionNotSupported(String),
@@ -284,10 +315,6 @@ pub enum Error {
     DateParsingError(#[from] chrono::ParseError),
     #[error("failed to parse leap second from \"{0}\"")]
     LeapSecondParsingError(String),
-    #[error("observation type code not recognized \"{0}\"")]
-    ObservationTypeCodeError(#[from] ObservationCodeError),
-    #[error("meteo data code not recognized \"{0}\"")]
-    MeteoDataCodeError(#[from] MeteoDataCodeError),
 }
 
 impl Default for RinexHeader {
@@ -296,7 +323,7 @@ impl Default for RinexHeader {
             version: version::Version::default(), 
             crinex: None,
             rinex_type: Type::default(),
-            constellation: Constellation::default(),
+            constellation: Some(Constellation::default()),
             program: String::from("Unknown"),
             run_by: String::from("Unknown"),
             station: String::from("Unknown"),
@@ -310,15 +337,16 @@ impl Default for RinexHeader {
             gps_utc_delta: None,
             // hardware
             rcvr: None,
-            // antenna
             ant: None,
+			sensors: None,
             coords: None, 
             // observations
             epochs: (None, None),
             wavelengths: None,
             obs_codes: None,
+			met_codes: None,
             // processing
-            rcvr_clock_offset_applied: None,
+            rcvr_clock_offset_applied: false,
             data_scaling: None,
             //ionospheric_corr: None,
             //gnsstime_corr: None,
@@ -339,7 +367,7 @@ impl std::str::FromStr for RinexHeader {
             line = lines.next()
                 .unwrap()
         }
-        // 'compact rinex'?
+        // 'compressed rinex'?
         let is_crinex = line.contains(CRINEX_MARKER_COMMENT);
         let crinex_infos: Option<CrinexInfo> = match is_crinex {
             false => None,
@@ -375,14 +403,19 @@ impl std::str::FromStr for RinexHeader {
         let (constellation_str, _) = remainder.trim().split_at(20);
 
         let rinex_type = Type::from_str(type_str.trim())?;
-        let constellation: Constellation;
+        let constellation: Option<Constellation>;
         
-        if type_str.trim().contains("GLONASS") {
+        if type_str.contains("GLONASS") {
             // special case, sometimes GLONASS NAV
             // drops the constellation field cause it's implied
-            constellation = Constellation::Glonass
-        } else {
-            constellation = Constellation::from_str(constellation_str.trim())?
+            constellation = Some(Constellation::Glonass)
+        } else if type_str.contains("METEOROLOGICAL DATA") {
+			// these files are not tied to a constellation system,
+			// therefore, do not have this field
+			constellation = None
+
+		} else { // regular files
+            constellation = Some(Constellation::from_str(constellation_str.trim())?)
         }
 
         let version = version::Version::from_str(version_str.trim())?;
@@ -477,15 +510,18 @@ impl std::str::FromStr for RinexHeader {
         let mut ant_coords : Option<rust_3d::Point3D> = None;
         let mut ant_hen    : Option<(f32,f32,f32)> = None;
         let mut rcvr       : Option<Rcvr>    = None;
+		let mut sensors    : Vec<Sensor> = Vec::with_capacity(3);
         // other
         let mut leap       : Option<LeapSecond> = None;
         let mut sampling_interval: Option<f32> = None;
-        let mut rcvr_clock_offset_applied: Option<bool> = None;
+        let mut rcvr_clock_offset_applied: bool = false;
         let mut coords     : Option<rust_3d::Point3D> = None;
         let mut epochs: (Option<gnss_time::GnssTime>, Option<gnss_time::GnssTime>) = (None, None);
         // (OBS) 
         let obs_nb_sat : u32 = 0;
-        let mut obs_codes  : Vec<String> = Vec::new();
+        let mut obs_codes  : HashMap<Constellation, Vec<String>> 
+            = HashMap::with_capacity(constellation::CONSTELLATION_LENGTH);
+		let mut met_codes  : Vec<String> = Vec::with_capacity(3);
 
         loop {
             /*
@@ -494,19 +530,30 @@ impl std::str::FromStr for RinexHeader {
                 to the number of records of the same epoch
                 following the 'epoch' record.
                 If may be used to skip appropriate number of data records if the event flags are not to be evaluated in detail
-
             */
             if line.contains("MARKER NAME") {
                 station = String::from(line.split_at(20).0.trim())
             } else if line.contains("MARKER NUMBER") {
                 station_id = String::from(line.split_at(20).0.trim()) 
             } else if line.contains("OBSERVER / AGENCY") {
-                let (obs, ag) = line.split_at(20);
+                let (content, _) = line.split_at(60);
+                let (obs, ag) = content.split_at(20);
                 observer = String::from(obs.trim());
                 agency = String::from(ag.trim())
 
             } else if line.contains("REC # / TYPE / VERS") {
                 rcvr = Some(Rcvr::from_str(line)?) 
+
+			} else if line.contains("SENSOR MOD/TYPE/ACC") {
+				let (content, _) = line.split_at(60);
+				let (model, rem) = content.split_at(20);
+				let (stype, rem) = rem.split_at(20+6);
+				let (accuracy, rem) = rem.split_at(7+4);
+				//println!("model \"{}\" stype \"{}\" accuracy \"{}\"", model, stype, accuracy);
+				let accuracy = f32::from_str(accuracy.trim())?;
+				let (physics, _) = rem.split_at(2);
+				sensors.push(Sensor::new(model.trim(),stype.trim(),accuracy,physics.trim()));
+				//println!("sensor {:#?}", sensors)
             
             } else if line.contains("ANT # / TYPE") {
                 ant = Some(Antenna::from_str(line)?)
@@ -530,7 +577,7 @@ impl std::str::FromStr for RinexHeader {
                 license = Some(String::from(lic.trim()))
             
             } else if line.contains("TIME OF FIRST OBS") {
-                let items: Vec<&str> = line.split_ascii_whitespace()
+                /*let items: Vec<&str> = line.split_ascii_whitespace()
                     .collect();
                 let (y, month, d, h, min, s, constel): (i32,u32,u32,u32,u32,f32,Constellation) =
                     (i32::from_str_radix(items[0].trim(),10)?,
@@ -541,10 +588,10 @@ impl std::str::FromStr for RinexHeader {
                     f32::from_str(items[5].trim())?,
                     Constellation::from_str(items[6].trim())?);
                 let utc = chrono::NaiveDate::from_ymd(y,month,d).and_hms(h,min,s as u32);
-                epochs.0 = Some(gnss_time::GnssTime::new(utc, constel)) 
+                epochs.0 = Some(gnss_time::GnssTime::new(utc, constel)) */
 
             } else if line.contains("TIME OF LAST OBS") {
-                let items: Vec<&str> = line.split_ascii_whitespace()
+               /* let items: Vec<&str> = line.split_ascii_whitespace()
                     .collect();
                 let (y, month, d, h, min, s, constel): (i32,u32,u32,u32,u32,f32,Constellation) =
                     (i32::from_str_radix(items[0].trim(),10)?,
@@ -555,11 +602,11 @@ impl std::str::FromStr for RinexHeader {
                     f32::from_str(items[5].trim())?,
                     Constellation::from_str(items[6].trim())?);
                 let utc = chrono::NaiveDate::from_ymd(y,month,d).and_hms(h,min,s as u32);
-                epochs.1 = Some(gnss_time::GnssTime::new(utc, constel)) 
+                epochs.1 = Some(gnss_time::GnssTime::new(utc, constel))*/ 
             
             } else if line.contains("WAVELENGTH FACT L1/2") {
-     //1     1                                                WAVELENGTH FACT L1/2
-            
+                //TODO
+
             } else if line.contains("APPROX POSITION XYZ") {
                 let items: Vec<&str> = line.split_ascii_whitespace()
                     .collect();
@@ -598,7 +645,7 @@ impl std::str::FromStr for RinexHeader {
             
             } else if line.contains("RCV CLOCK OFFS APPL") {
                 let ok_str = line.split_at(20).0.trim();
-                rcvr_clock_offset_applied = Some(i32::from_str_radix(ok_str, 10)? != 0)
+                rcvr_clock_offset_applied = i32::from_str_radix(ok_str, 10)? != 0
 
             } else if line.contains("# OF SATELLITES") {
                 // will always appear prior PRN/#OBS
@@ -632,62 +679,89 @@ impl std::str::FromStr for RinexHeader {
                 // Rinex V < 3 : old fashion obs data
                 // ⚠ ⚠ could either be observation or meteo data
                 let (rem, _) = line.split_at(60);
-                match rinex_type {
-                    Type::ObservationData => {
-                        let (_, mut rem) = rem.split_at(6);
-                        loop {
-                            let (code, r) = rem.split_at(6);
-                            if code.trim().len() == 0 {
-                                break
-                            }
-                            obs_codes.push(String::from(code.trim()));
-                            if r.len() == 0 {
-                                break
-                            }
-                            rem = r
-                        }
+				let (n_codes, mut line) = rem.split_at(6);
+				let n_codes = u8::from_str_radix(n_codes.trim(), 10)?;
+				let n_lines : usize = num_integer::div_ceil(n_codes, 9).into();
+                let mut codes : Vec<String> = Vec::new();
+
+				for i in 0..n_lines {
+					let content : Vec<&str> = line.split_ascii_whitespace()
+						.collect();
+					for j in 0..content.len() { 
+                    	codes.push(String::from(content[j].trim()));
+					}
+					if i < n_lines-1 { // takes more than one line
+						line = lines.next() // --> need to grab new content
+							.unwrap();
+						line = line.split_at(60).0 // remove comments
+					}
+				}
+
+                // build code map 
+                // to be used later on when parsing payload
+                match constellation {
+                    Some(Constellation::Mixed) => {
+						// Multi constell:
+						//  trick to later identify, in all cases
+                        let constells : Vec<Constellation> =
+						Vec::from([
+                            Constellation::GPS,
+                            Constellation::Glonass,
+                            Constellation::Galileo,
+                            Constellation::Beidou,
+                            Constellation::Sbas,
+                            Constellation::QZSS,
+                        ]);
+                		for i in 0..constells.len() {
+                    		obs_codes.insert(constells[i], codes.clone());
+                		}
                     },
-                    Type::MeteorologicalData => {
-                        let (_, mut rem) = rem.split_at(6);
-                        loop {
-                            let (code, r) = rem.split_at(6);
-                            if code.trim().len() == 0 {
-                                break
-                            }
-                            obs_codes.push(String::from(code.trim()));
-                            if r.len() == 0 {
-                                break
-                            }
-                            rem = r
-                        }
-                    },
-                    _ => {},
-                }
+                    Some(c) => {
+						// Single constellation system
+						obs_codes.insert(c, codes.clone());
+					},
+					_ => {
+						// this is a meteo file,
+						// meteo observations are not tied to a specific
+						// constellation system
+						for i in 0..codes.len() {
+							met_codes.push(codes[i].clone())
+						}
+					},
+                };
+
             } else if line.contains("SYS / # / OBS TYPES") {
-                //let content = line.split_at(60).0;
-/*G   22 C1C L1C D1C S1C C1W S1W C2W L2W D2W S2W C2L L2L D2L  SYS / # / OBS TYPES
-       S2L C5Q L5Q D5Q S5Q C1L L1L D1L S1L                  SYS / # / OBS TYPES
-E   20 C1C L1C D1C S1C C6C L6C D6C S6C C5Q L5Q D5Q S5Q C7Q  SYS / # / OBS TYPES
-       L7Q D7Q S7Q C8Q L8Q D8Q S8Q                          SYS / # / OBS TYPES
-S    8 C1C L1C D1C S1C C5I L5I D5I S5I                      SYS / # / OBS TYPES
-R   20 C1C L1C D1C S1C C1P L1P D1P S1P C2P L2P D2P S2P C2C  SYS / # / OBS TYPES
-       L2C D2C S2C C3Q L3Q D3Q S3Q                          SYS / # / OBS TYPES
-C   20 C1P L1P D1P S1P C5P L5P D5P S5P C2I L2I D2I S2I C7I  SYS / # / OBS TYPES
-       L7I D7I S7I C6I L6I D6I S6I                          SYS / # / OBS TYPES
-I    4 C5A L5A D5A S5A                                      SYS / # / OBS TYPES*/
-                // Rinex::ObservationData 
-                // types of observation for specified constellation
-                
-                // grab Constellation info:
-                //   [+] if constellation info exists: new entry
-                //   [+] otherwise: adding some more to a previous entry
-                let (sys, rem) = line.split_at(3);
-                let (nb, rem) = rem.split_at(3);
-                if sys.trim().len() > 0 {
-                    println!("SYS \"{}\" NB \"{}\" REM \"{}\"", sys.trim(), nb.trim(), rem.trim());
-                } else {
-                    println!("REM \"{}\"", rem.trim());
+                // modern obs code descriptor 
+                let (line, _) = line.split_at(60); // remove header
+                let (identifier, rem) = line.split_at(1);
+                let (n_codes, mut line) = rem.split_at(5);
+                let n_codes = u8::from_str_radix(n_codes.trim(), 10)?;
+				let n_lines : usize = num_integer::div_ceil(n_codes, 13).into(); 
+                let mut codes : Vec<String> = Vec::with_capacity(n_codes.into());
+                let constell : constellation::Constellation = match identifier {
+                    "G" => constellation::Constellation::GPS,
+                    "R" => constellation::Constellation::Glonass,
+                    "J" => constellation::Constellation::QZSS,
+                    "E" => constellation::Constellation::Galileo,
+                    "C" => constellation::Constellation::Beidou,
+                    "S" => constellation::Constellation::Sbas,
+                    _ => continue, // should never happen 
+                };
+
+                for i in 0..n_lines {
+					let content : Vec<&str> = line.split_ascii_whitespace()
+						.collect();
+					for j in 0..content.len() { 
+                    	codes.push(String::from(content[j].trim()));
+					}
+					if i < n_lines-1 { // takes more than one line
+						line = lines.next() // --> need to grab new content
+							.unwrap();
+						line = line.split_at(60).0 // remove comments
+					}
                 }
+                obs_codes.insert(constell, codes);
+            
             } else if line.contains("SIGNAL STRENGHT UNIT") {
                 //TODO
             } else if line.contains("INTERVAL") {
@@ -743,10 +817,23 @@ I    4 C5A L5A D5A S5A                                      SYS / # / OBS TYPES*
             }
         }
 
-        let obs_codes: Option<Vec<String>> = match obs_codes.len() {
-            0 => None,
-            _ => Some(obs_codes)
+		let sensors : Option<Vec<Sensor>> = match sensors.len() > 0 {
+			true => {
+				// identified some sensors
+				Some(sensors)
+			},
+			false => None
+		};
+
+        let obs_codes: Option<HashMap<Constellation, Vec<String>>> = match obs_codes.is_empty() {
+            true => None,
+            false => Some(obs_codes),
         };
+
+		let met_codes : Option<Vec<String>> = match met_codes.is_empty() {
+			true => None,
+			false => Some(met_codes),
+		};
         
         Ok(RinexHeader{
             version: version,
@@ -764,15 +851,14 @@ I    4 C5A L5A D5A S5A                                      SYS / # / OBS TYPES*
             station_url,
             rcvr, 
             ant, 
+			sensors,
             leap,
             coords: coords,
-            // ?
             wavelengths: None,
             gps_utc_delta: None,
-            // observations
-            epochs: epochs,
+            epochs,
             obs_codes,
-            // processing
+			met_codes,
             sampling_interval: sampling_interval,
             data_scaling: None,
             //ionospheric_corr: None,
@@ -783,24 +869,24 @@ I    4 C5A L5A D5A S5A                                      SYS / # / OBS TYPES*
 }
 
 impl RinexHeader {
-    /// Returns true if self is a `Compact RINEX`
+    /// Returns true if self is a `Compressed RINEX`
     pub fn is_crinex (&self) -> bool { self.crinex.is_some() }
 
-    /// Returns `Compact RINEX` version (if any) 
+    /// Returns Compressed `RINEX` version (if any) 
     pub fn get_crinex_version (&self) -> Option<&str> { 
         match &self.crinex {
             Some(crinex) => Some(&crinex.version),
             _ => None,
         }
     }
-    /// Returns `Compact RINEX` prog (if any) 
+    /// Returns `RINEX` compression program name (if any) 
     pub fn get_crinex_prog (&self) -> Option<&str> { 
         match &self.crinex {
             Some(crinex) => Some(&crinex.prog),
             _ => None,
         }
     }
-    /// Returns `Compact RINEX` date (if any) 
+    /// Returns `RINEX` compression date (if any) 
     pub fn get_crinex_date (&self) -> Option<chrono::NaiveDateTime> { 
         match &self.crinex {
             Some(crinex) => Some(crinex.date),
