@@ -82,7 +82,10 @@ impl std::str::FromStr for Type {
 /// `Rinex` describes a `RINEX` file
 #[derive(Debug)]
 pub struct Rinex {
+    /// `header` field contains general information
     pub header: header::RinexHeader,
+    /// `record` contains `RINEX` file body
+    /// and is type and constellation dependent 
     pub record: Option<record::Record>,
 }
 
@@ -130,10 +133,6 @@ impl Rinex {
         Ok((String::from(header),String::from(body)))
     }
 
-    // Returns Record nth' entry
-    //pub fn get_record_nth (&self, nth: usize) 
-    //    -> &std::collections::HashMap<String, record::RecordItem> { &self.record[nth] }
-
     /// Retruns true if this is an NAV rinex
     pub fn is_navigation_rinex (&self) -> bool { self.header.rinex_type == Type::NavigationMessage }
     /// Retruns true if this is an OBS rinex
@@ -156,25 +155,65 @@ impl Rinex {
     }
 }
 
+#[cfg(test)]
 mod test {
     use super::*;
     #[test]
     /// Tests `Rinex` constructor against all known test resources
     fn test_rinex_constructor() {
-        // open test resources
-        let test_resources = std::path::PathBuf::from(
-            env!("CARGO_MANIFEST_DIR").to_owned() + "/data");
-        // walk test resources
-        for entry in std::fs::read_dir(test_resources)
-            .unwrap() {
-            let entry = entry
-                .unwrap();
-            let path = entry.path();
-            if !path.is_dir() { // only files..
-                let fp = std::path::Path::new(&path);
-                let rinex = Rinex::from_file(&fp);
-                assert_eq!(rinex.is_err(), false);
-                println!("File: {:?}\n{:#?}", &fp, rinex)
+        let test_dir = env!("CARGO_MANIFEST_DIR").to_owned() + "/data";
+        let types = vec![
+			"NAV",
+			"OBS",
+			"MET"
+		];
+        for t in types {
+            let versions = vec![
+				"V2",
+				"V3"
+			];
+            for v in versions {
+                let dir_path = std::path::PathBuf::from(
+                    test_dir.to_owned() + "/"+t + "/"+v
+                );
+                for entry in std::fs::read_dir(dir_path)
+                    .unwrap() {
+                    let entry = entry
+                        .unwrap();
+                    let path = entry.path();
+					let is_hidden = entry.file_name()
+						.to_str()
+							.unwrap()
+							.starts_with(".");
+                    if !path.is_dir() && !is_hidden { // only relevant files..
+                        let fp = std::path::Path::new(&path);
+                        let rinex = Rinex::from_file(&fp);
+                        assert_eq!(rinex.is_err(), false);
+						let rinex = rinex.unwrap();
+                        println!("File: {:?}\n{:#?}", &fp, rinex);
+						match t {
+							"NAV" => {
+                                // NAV files sanity checks
+                                assert_eq!(rinex.is_navigation_rinex(), true);
+                            },
+							"OBS" => {
+                                // OBS files sanity checks
+                                assert_eq!(rinex.is_observation_rinex(), true);
+                                assert_eq!(rinex.header.obs_codes.is_some(), true);
+                                if rinex.header.rcvr_clock_offset_applied {
+                                    // epochs should always have a RCVR clock offset
+                                    // test that with iterator
+                                }
+                            },
+							"MET" => {
+                                // METEO files sanity checks
+                                assert_eq!(rinex.is_meteo_rinex(), true);
+                                assert_eq!(rinex.header.met_codes.is_some(), true);
+                            },
+							_ => {},
+						}
+                    }
+                }
             }
         }
     }
